#!/bin/bash

# Copyright (C) since 2015 NTT DATA Corporation
# 
# This program is free software; you can redistribute it and/or
# Modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details

########################################
#  Function
########################################

function print_usage() {
	name=`basename $0`
	cat <<EOF
Usage:
	./$name [-w PASSWORD] import FILE_ID FILE_PATH OWNER_ROLE_ID
	./$name [-w PASSWORD] export FILE_ID
	./$name [-w PASSWORD] list

Parameters:
	FILE_ID       : Infra file ID
	FILE_PATH     : Infra file path
	OWNER_ROLE_ID : Owner role ID (e.g. ALL_USERS, ADMINISTRATORS)

Options:
	-w PASSWORD	specify password for Hinemos RDBM Server

EOF
}

function connect_db() {
	if [ -z ${HINEMOS_PASSWD} ]; then
		stty -echo
		read -p "PostgreSQL Password (default hinemos) : " HINEMOS_PASSWD
		stty echo
		echo
	fi
	query_sql "SELECT 1;" > /dev/null
	EXIT_CODE=$?
	if [ ${EXIT_CODE} -ne 0 ] ; then
		echo "Failed to access PostgreSQL" 1>&2
		echo "exit code ${EXIT_CODE}" 1>&2
		exit ${EXIT_CODE}
	fi
}

function exec_sql() {
	FUNC_RET=""

	PSQL=$SCRIPT_DIR/../../postgresql/bin/psql
	HINEMOS_USER=hinemos
	PORT=24001

	FUNC_RET=`PGPASSWORD=${HINEMOS_PASSWD} $PSQL -p $PORT -U ${HINEMOS_USER} -w -t -A -c "$1"`
}

function query_sql() {
	FUNC_RET=""

	PSQL=$SCRIPT_DIR/../../postgresql/bin/psql
	HINEMOS_USER=hinemos
	PORT=24001

	PGPASSWORD=${HINEMOS_PASSWD} $PSQL -p $PORT -U ${HINEMOS_USER} -w -c "$1"
}

function import() {
	if [ "$1" == "" ] || [ "$2" == "" ] || [ "$3" == "" ]
	then
		print_usage
		exit 1
	fi

	connect_db

	FILE_ID=$1
	FILE_PATH=$2
	FILE_NAME=`basename $FILE_PATH`
	OWNER_ROLE_ID=$3

	if [ ! -e $FILE_PATH ]
	then
		echo "The file '$FILE_PATH' doesn't exist."
		exit 1
	fi

	#create temp file to store hex string
	HEX_FILE="$FILE_NAME.hex."`date +"%s"`
	xxd -p $FILE_PATH | tr -d '\n' > $HEX_FILE

	#create a record
	exec_sql "begin;
			insert into setting.cc_infra_file(file_id, file_name, owner_role_id, create_user_id, create_datetime, modify_user_id, modify_datetime)
				values('$FILE_ID', '$FILE_NAME', '$OWNER_ROLE_ID', 'hinemos', now(), 'hinemos', now());
			insert into binarydata.cc_infra_file_content(file_id) values('$FILE_ID');
			end;"
	if [ $? -ne 0 ]
	then
		echo "The new record cannot be inserted into table cc_infra_file. Please check related error message."

		#remove temp file
		rm -f $HEX_FILE

		exit 1
	fi

	#create temp table
	exec_sql "create table binarydata.hexdump(hex text);"

	#copy file content to temp table
	exec_sql "\\copy binarydata.hexdump from '$HEX_FILE';"

	#update content in cc_infra_file
	exec_sql "update binarydata.cc_infra_file_content
			set file_content=(select decode(hex, 'hex') from binarydata.hexdump)
			where file_id='$FILE_ID'"
	
	#remove temp table
	exec_sql "drop table binarydata.hexdump;"

	#remove temp file
	rm -f $HEX_FILE

	echo "The file '$FILE_PATH' is imported."
}

function export() {
	if [ "$1" == "" ]
	then
		print_usage
		exit 1
	fi

	connect_db

	FILE_ID=$1

	exec_sql "select file_name from setting.cc_infra_file where file_id='$FILE_ID';"
	FILE_NAME="$FUNC_RET"
	if [ "$FILE_NAME" == "" ]
	then
		echo "It failed to find record with file_id '$FILE_ID' in table cc_infra_file."
		echo "Please check whether the file_id '$FILE_ID' is correct or not."
		exit 1
	fi

	if [ -e "$FILE_NAME" ]
	then
		echo "The file '$FILE_NAME' has existed. Please rename the file before exporting."
		exit 1
	fi

	#copy content to temp file
	HEX_FILE="$FILE_NAME.hex."`date +"%s"`
	exec_sql "\\copy (select encode(file_content, 'hex') from binarydata.cc_infra_file_content where file_id='$FILE_ID')
			to '$HEX_FILE';"
	
	#convert the content of temp file to binary format
	xxd -p -r $HEX_FILE > $FILE_NAME

	#remove temp file
	rm -f $HEX_FILE

	echo "The infra file '${FILE_ID}' is exported to file '$FILE_NAME'."
}

function list() {
	connect_db
	query_sql "select file_id, file_name, owner_role_id, modify_datetime from setting.cc_infra_file;"
}

########################################
#  Check parameters
########################################
ACTION=
ARGS=()
while [ -n "$1" ]; do
	case "$1" in
		-h|--help)
			print_usage
			exit 0
			;;
		-w)
			shift 1
			HINEMOS_PASSWD="$1"
			;;
		import|export|list)
			ACTION=$1
			;;
		*)
			ARGS+=("$1")
	esac
	shift 1
done

########################################
#  Main
########################################


SCRIPT_DIR=`dirname $0`

if [ "$ACTION" = "import" ] && [ ${#ARGS[@]} -eq 3 ]; then
	import "${ARGS[0]}" "${ARGS[1]}" "${ARGS[2]}"
elif [ "$ACTION" = "export" ] && [ ${#ARGS[@]} -eq 1 ]; then
	export "${ARGS[0]}"
elif [ "$ACTION" = "list" ] && [ ${#ARGS[@]} -eq 0 ]; then
	list
else
	print_usage
	exit 1
fi

exit 0

