#!/bin/bash

# Copyright (C) 2015 NTT DATA Corporation
# 
# This program is free software; you can redistribute it and/or
# Modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details

## Hinemos env
SCRIPT_DIR=$(cd $(dirname $0);pwd)
. ${SCRIPT_DIR}/../../hinemos.cfg
. ${SCRIPT_DIR}/../hinemos_utility.sh

########################################
#  Local Variable
########################################
FILENAME_POSTFIX=`date '+%Y-%m-%d_%H-%M-%S'`
SUMMARY_FILE="/opt/hinemos/var/log/hinemos_manager_summary.${FILENAME_POSTFIX}"

JMX_HOST=${JMX_HOST:-127.0.0.1}
JMX_PORT=${JMX_PORT:-7100}
JMX_USER=
JMX_PASS=

########################################
# Function
########################################

function usage {
	cat <<EOF
Usage:
	$0 -v   : Get information about HINEMOS SUMMARY
	$0 -vv  : Get information about HINEMOS SUMMARY + OS INFORMATION
	$0 -vvv : Get information about DETAIL
	$0 -o   : Get information about DETAIL + Log about HINEMOS MANAGER
	$0 -t   : Get thread dump
	$0 -md5 : Get information about files's md5sum

Options:
	-w PASSWORD  specify password for Hinemos RDBM Server
	-u USERNAME  specify username for JMX authentication
	-p PASSWORD  specify password for JMX authentication

EOF
	return 0
} # function usage


function hinemos {
	## Version
	VER=`cat ${HINEMOS_HOME}/_version`
	echo "Installed Hinemos ver : ${VER}" | tee -a ${SUMMARY_FILE}
	echo "" | tee -a ${SUMMARY_FILE}

	## init valiable
	HOST="localhost"
	PGPORT=24001
	PSQLBIN="${PG_HOME}/bin/psql"
	PG_FACILITY="true"
	COLUMN_COLLECTOR_PROFILE_INTERVAL="interval_sec"
	VALUE_TRIGGER_NAME_PROCESS="MON_PRC"
	
	DBNAME=hinemos
	USER=hinemos
	FILENAME="${HINEMOS_HOME}/var/log/db-summary.`date '+%Y-%m-%d_%T'`"
	TMPFILE="${HINEMOS_HOME}/var/log/db-summary.tmp"
	
	PSQLCMD="${PSQLBIN} -p ${PGPORT} -U ${USER} -d ${DBNAME} "
	PSQLCMD_NA="${PSQLBIN} -p ${PGPORT} -U ${USER} -d ${DBNAME} -A -t "

	## postgresql password
	## postgresql connect check
	if [ -z ${PGPASSWORD} ]; then
		stty -echo
		read -p "PostgreSQL Password (default hinemos) : " PASSWD
		stty echo
		echo
		if [ "${PASSWD}" != "" ] ; then
			export PGPASSWORD=${PASSWD}
		fi
	fi
	RESULT=`${PSQLCMD_NA} -c "SELECT 1;"`
	EXIT_CODE=$?
	if [ ${EXIT_CODE} -ne 0 ] ; then
		echo "Failed to access PostgreSQL" 1>&2
		echo "exit code ${EXIT_CODE}" 1>&2
	
		rm ${SUMMARY_FILE}
	
		exit ${EXIT_CODE}
	fi

	## qrtz_simple_trigger summary
	echo "### Simple trigger summary ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		a.trigger_state,
		a.trigger_group,
		b.repeat_interval,
		count(*)
	FROM
		setting.qrtz_triggers as a,
		setting.qrtz_simple_triggers as b
	WHERE
		a.trigger_name = b.trigger_name
	GROUP BY
		a.trigger_state,
		a.trigger_group,
		b.repeat_interval
	ORDER BY
		a.trigger_state,
		a.trigger_group,
		b.repeat_interval
" | tee -a ${SUMMARY_FILE}

	## qrtz_cron_trigger summary
	echo "### Cron trigger summary ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		a.trigger_state,
		a.trigger_group,
		b.cron_expression,
		count(*)
	FROM
		setting.qrtz_triggers as a,
		setting.qrtz_cron_triggers as b
	WHERE
		a.trigger_name = b.trigger_name
	GROUP BY
		a.trigger_state,
		a.trigger_group,
		b.cron_expression -- fire_duration_minutes
	ORDER BY
		a.trigger_state,
		a.trigger_group
" | tee -a ${SUMMARY_FILE}

	## JOB-net
	echo "### Job-net summary ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		CASE job_type
			WHEN 0 THEN 'JU'
			WHEN 1 THEN 'JN'
			WHEN 2 THEN 'JB'
			WHEN 3 THEN 'FT'
			WHEN 5 THEN 'RF'
			ELSE        '??'
		END AS type,
		count(*)
	FROM
		setting.cc_job_mst
	GROUP BY
		job_type
	ORDER BY
		job_type
" | tee -a ${SUMMARY_FILE}

	## running job
	echo "### Running Job count ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT count(*)
	FROM
		log.cc_job_session s, log.cc_job_session_job j
	WHERE
		s.session_id = j.session_id AND s.jobunit_id = j.jobunit_id AND
		 s.job_id = j.job_id AND status / 100 != 3
" | tee -a ${SUMMARY_FILE}

	## Hinemos Property
	echo "### hinemos property summary ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		key,
		CASE
			WHEN LOWER(key) LIKE '%password%' then '********'
			ELSE value_string
		END AS value_string_masked,
		value_numeric,
		value_boolean,
		value_type,
		description,
		owner_role_id,
		create_user_id,
		create_datetime,
		modify_user_id,
		modify_datetime
	FROM
		setting.cc_hinemos_property
	ORDER BY
		key
" | tee -a ${SUMMARY_FILE}


	## log table size
	echo "### log table summary ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		relname,
		reltuples,
		pg_relation_size(CONCAT(ns.nspname,'.',relname)::regclass)
	FROM pg_class
	LEFT JOIN pg_namespace ns ON ns.oid = pg_class.relnamespace
	WHERE
		ns.nspname LIKE 'log' AND relname IN (
			'cc_calculated_data',
			'cc_event_log',
			'cc_job_session',
			'cc_job_session_job',
			'cc_job_session_node'
		)
" | tee -a ${SUMMARY_FILE}

	## large table
	echo "### more than 100 record table ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		relname,
		reltuples,
		pg_relation_size(CONCAT(ns.nspname,'.',relname)::regclass)
	FROM
		pg_class
	LEFT JOIN pg_namespace AS ns ON ns.oid = pg_class.relnamespace
	WHERE
		relkind = 'r' AND
		ns.nspname IN ('setting', 'binarydata', 'log') AND
		reltuples > 100
	ORDER BY
		reltuples DESC
" | tee -a ${SUMMARY_FILE}

	## PostgreSQL info
	echo "### rdbms repository info ###" | tee -a ${SUMMARY_FILE}
	SCOPE=`${PSQLCMD_NA} -c "SELECT count(*) FROM setting.cc_cfg_facility WHERE facility_type = 0"`
	REALNODE=`${PSQLCMD_NA} -c "SELECT count(*) FROM setting.cc_cfg_facility WHERE facility_type = 1"`
	VIRTNODE=`${PSQLCMD_NA} -c " SELECT count(*) FROM setting.cc_cfg_facility_relation WHERE parent_facility_id IN ( SELECT facility_id FROM setting.cc_cfg_facility WHERE facility_type = 0 )"`
	DISK=`${PSQLCMD_NA} -c "SELECT count(*) FROM setting.cc_cfg_node_device WHERE device_type = 'disk'"`
	NIC=`${PSQLCMD_NA} -c "SELECT count(*) FROM setting.cc_cfg_node_device WHERE device_type = 'nic'"`
	FS=`${PSQLCMD_NA} -c "SELECT count(*) FROM setting.cc_cfg_node_filesystem"`

	echo "Scope      : ${SCOPE}" | tee -a ${SUMMARY_FILE}
	echo "Node       : ${VIRTNODE} (RealNode : ${REALNODE})" | tee -a ${SUMMARY_FILE}
	echo "DISK       : ${DISK}" | tee -a ${SUMMARY_FILE}
	echo "NIC        : ${NIC}" | tee -a ${SUMMARY_FILE}
	echo "FileSystem : ${FS}" | tee -a ${SUMMARY_FILE}
	echo "" | tee -a ${SUMMARY_FILE}

	echo "### Node Platform ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		platform_family,
		sub_platform_family,
		cloud_service,
		cloud_resource_type,
		count(*) count
	FROM
		setting.cc_cfg_node
	GROUP BY
		platform_family,
		sub_platform_family,
		cloud_service,
		cloud_resource_type
	ORDER BY
		count DESC;
" | tee -a ${SUMMARY_FILE}

	## pg_stat_database
	echo "### pg_stat_database ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		*
	FROM
		pg_stat_database
" | tee -a ${SUMMARY_FILE}

	## pg_stat_user_tables
	echo "### pg_stat_user_tables ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		*
	FROM
		pg_stat_user_tables
	ORDER BY
		seq_scan + seq_tup_read + idx_scan + idx_tup_fetch + n_tup_ins +
		n_tup_upd + n_tup_del + n_tup_hot_upd + n_live_tup + n_dead_tup DESC
" | tee -a ${SUMMARY_FILE}

	## pg_stat_user_indexes
	echo "### pg_stat_user_indexes ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		*
	FROM
		pg_stat_user_indexes
	ORDER BY
		idx_scan + idx_tup_read + idx_tup_fetch DESC
" | tee -a ${SUMMARY_FILE}

	## pg_stat_activity
	echo "### pg_stat_activity ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		*
	FROM
		pg_stat_activity
	ORDER BY query_start
" | tee -a ${SUMMARY_FILE}

	## pg_statio_user_tables
	echo "### pg_statio_user_tables ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		*
	FROM
		pg_statio_user_tables
	ORDER BY
		heap_blks_read + heap_blks_hit + idx_blks_read + idx_blks_hit DESC
" | tee -a ${SUMMARY_FILE}

	## pg_statio_user_indexes
	echo "### pg_statio_user_indexes ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		*
	FROM
		pg_statio_user_indexes
	ORDER BY
		idx_blks_read + idx_blks_hit DESC
" | tee -a ${SUMMARY_FILE}

	## txid_current
	echo "### txid_current ###" | tee -a ${SUMMARY_FILE}
	${PSQLCMD} -c "
	SELECT
		txid_current()
" | tee -a ${SUMMARY_FILE}

} # function hinemos

function os {
	## cat /etc/redhat-release
	echo "### cat /etc/redhat-release ###" | tee -a ${SUMMARY_FILE}
	cat /etc/redhat-release | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}

	## /proc/PID/environ
	echo "### /proc/PID/environ ###" | tee -a ${SUMMARY_FILE}
	xargs -n 1 -0 < /proc/`cat ${JVM_PID_FILE}`/environ | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}

	## systemctl list-dependencies | grep hinemos
	echo "### systemctl list-dependencies | grep hinemos ###" | tee -a ${SUMMARY_FILE}
	systemctl list-dependencies | grep hinemos | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}

	## systemctl status hinemos_manager
	echo "### systemctl status hinemos_manager" | tee -a ${SUMMARY_FILE}
	systemctl status hinemos_manager | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}

	## rpm -Va hinemos-manager
	echo "### rpm -Va hinemos-manager ###" | tee -a ${SUMMARY_FILE}
	rpm -Va hinemos-manager | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}

	## rpm -qa
	echo "### rpm -qa ###" | tee -a ${SUMMARY_FILE}
	rpm -qa | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}

	## cpuinfo
	echo "### /proc/cpuinfo ###" | tee -a ${SUMMARY_FILE}
	cat /proc/cpuinfo | tee -a ${SUMMARY_FILE}
	echo -e "" | tee -a ${SUMMARY_FILE}
	
	## meminfo
	echo "### /proc/meminfo ###" | tee -a ${SUMMARY_FILE}
	cat /proc/meminfo | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
	
	## sar -P ALL
	echo "### sar -P ALL ###" | tee -a ${SUMMARY_FILE}
	sar -P ALL | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
	
	## sar -r
	echo "### sar -r ###" | tee -a ${SUMMARY_FILE}
	sar -r | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
	
	## sar -n DEV
	echo "### sar -n DEV ###" | tee -a ${SUMMARY_FILE}
	sar -n DEV | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
	
	## sar -n EDEV
	echo "### sar -n EDEV ###" | tee -a ${SUMMARY_FILE}
	sar -n EDEV | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
	
	## vmstat -n
	echo "### vmstat -n ###" | tee -a ${SUMMARY_FILE}
	vmstat -n | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
	
	## iostat -d -x
	echo "### iostat -d -x ###" | tee -a ${SUMMARY_FILE}
	iostat -d -x | tee -a ${SUMMARY_FILE}
	echo -e "" | tee -a ${SUMMARY_FILE}
	
	## ps -H auxwww
	echo "### ps -H auxwww ###" | tee -a ${SUMMARY_FILE}
	ps -H auxwww | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
	
	## ps -L auxH
	echo "### ps -L auxH ###" | tee -a ${SUMMARY_FILE}
	ps -L auxH | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}

	netstat --version > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		## netstat -s
		echo "### netstat -s ###" | tee -a ${SUMMARY_FILE}
		netstat -s | tee -a ${SUMMARY_FILE}
		echo -e "\n" | tee -a ${SUMMARY_FILE}

		## netstat -anpo
		echo "### netstat -anpo ###" | tee -a ${SUMMARY_FILE}
		netstat -anpo | tee -a ${SUMMARY_FILE}
		echo -e "\n" | tee -a ${SUMMARY_FILE}
	else
		## ss -s
		echo "### ss -s ###" | tee -a ${SUMMARY_FILE}
		ss -s | tee -a ${SUMMARY_FILE}
		echo -e "\n" | tee -a ${SUMMARY_FILE}

		## ss -anpo
		echo "### ss -anpot ###" | tee -a ${SUMMARY_FILE}
		ss -anpot | tee -a ${SUMMARY_FILE}
		echo -e "\n" | tee -a ${SUMMARY_FILE}
		echo "### ss -anpou ###" | tee -a ${SUMMARY_FILE}
		ss -anpou | tee -a ${SUMMARY_FILE}
		echo -e "\n" | tee -a ${SUMMARY_FILE}
	fi

	## df
	echo "### df ###" | tee -a ${SUMMARY_FILE}
	df | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
	
	## df -i
	echo "### df -i ###" | tee -a ${SUMMARY_FILE}
	df -i | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
	
	## uptime
	echo "### uptime ###" | tee -a ${SUMMARY_FILE}
	uptime | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
	
	## /proc/slabinfo
	echo "### /proc/slabinfo ###" | tee -a ${SUMMARY_FILE}
	cat /proc/slabinfo | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
	
	## /proc/vmstat
	echo "### /proc/vmstat ###" | tee -a ${SUMMARY_FILE}
	cat /proc/vmstat | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}

} # function os


function md5sum {
	## md5sum
	echo "### md5sum ###" | tee -a ${SUMMARY_FILE}
	cd ${HINEMOS_HOME}
	find `ls --ignore='var'` -type f -print0 | xargs -0 md5sum | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}
}

function detail {
	## lsof
	echo "### lsof ###" | tee -a ${SUMMARY_FILE}
	lsof | tee -a ${SUMMARY_FILE}
	echo -e "\n" | tee -a ${SUMMARY_FILE}

	# manager_cli
	CLI_CMD=${HINEMOS_HOME}/sbin/manager_cli

	if [ ! -z ${JMX_HOST} ];then
		CLI_CMD="${CLI_CMD} -H ${JMX_HOST}"
	fi
	if [ ! -z ${JMX_PORT} ];then
		CLI_CMD="${CLI_CMD} -P ${JMX_PORT}"
	fi
	if [ ! -z ${JMX_USER} ];then
		CLI_CMD="${CLI_CMD} -u ${JMX_USER}"
	fi
	if [ ! -z ${JMX_PASS} ];then
		CLI_CMD="${CLI_CMD} -p ${JMX_PASS}"
	fi

	echo "### manager_cli (Hinemos Agent Info) ###" | tee -a ${SUMMARY_FILE}
	${CLI_CMD} -a ValidAgentStr | tee -a ${SUMMARY_FILE}

	echo "### manager_cli (Scheduler Info) ###" | tee -a ${SUMMARY_FILE}
	${CLI_CMD} -a SchedulerInfoStr | tee -a ${SUMMARY_FILE}

	echo "### manager_cli (Poller Info) ###" | tee -a ${SUMMARY_FILE}
	${CLI_CMD} -a PollerInfoStr | tee -a ${SUMMARY_FILE}

	echo "### manager_cli (SharedTable Info) ###" | tee -a ${SUMMARY_FILE}
	${CLI_CMD} -a SharedTableInfoStr | tee -a ${SUMMARY_FILE}

	echo "### manager_cli (SelfCheck LastFireTime) ###" | tee -a ${SUMMARY_FILE}
	${CLI_CMD} -a SelfCheckLastFireTimeStr | tee -a ${SUMMARY_FILE}

	echo "### manager_cli (AsynWorker Statistics) ###" | tee -a ${SUMMARY_FILE}
	${CLI_CMD} -a AsyncWorkerStatistics | tee -a ${SUMMARY_FILE}

	echo "### manager_cli (Syslog Statistics) ###" | tee -a ${SUMMARY_FILE}
	${CLI_CMD} -a SyslogStatistics | tee -a ${SUMMARY_FILE}

	echo "### manager_cli (SnmpTrap Statistics) ###" | tee -a ${SUMMARY_FILE}
	${CLI_CMD} -a SnmpTrapStatistics | tee -a ${SUMMARY_FILE}

	echo "### manager_cli (Job Queue Info) ###" | tee -a ${SUMMARY_FILE}
	${CLI_CMD} -a JobQueueStr | tee -a ${SUMMARY_FILE}

	echo -e "\n" | tee -a ${SUMMARY_FILE}

	md5sum
} # function detail


function threaddump {

	DUMP_FILENAME="threaddump.${FILENAME_POSTFIX}"
	DUMP_INFO_FILENAME="threaddump_info.${FILENAME_POSTFIX}"
	
	echo "### Get thread dump  ###"
	
	while [ 1 ]
	do
		sleep 0.5 && read -p "input interval (sec) : " INPUT
		case ${INPUT} in
			[0-9]*)
				INTERVAL=${INPUT}
				break;
				;;
			*)
				echo "Your input is invalid. Please input again."
				;;
		esac
	done
	
	
	while [ 1 ]
	do
		sleep 0.5 && read -p "input count : " INPUT
		case ${INPUT} in
			[0-9]*)
				COUNT=${INPUT}
				break;
				;;
			*)
				echo "Your input is invalid. Please input again."
				;;
		esac
	done
	
	
	NUM=0
	while [ 1 ] ;
	do
	
		NUM=`expr ${NUM} + 1`
		echo -n "getting thread dump ${NUM}/${COUNT} ... "  | tee -a ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
		echo -n `date`  | tee -a ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
		kill -3 `cat ${HINEMOS_HOME}/var/run/jvm.pid`
		echo " [OK] "  | tee -a ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
		echo -e "\n" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}

		netstat --version > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			echo "### netstat -s ###" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
			netstat -s >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
			echo -e "\n" >> ${HINEMOS_HOME}/var/log//${DUMP_INFO_FILENAME}

			echo "### netstat -anpo ###" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
			netstat -anpo >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
			echo -e "\n" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
		else
			echo "### ss -s ###" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
			ss -s >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
			echo -e "\n" >> ${HINEMOS_HOME}/var/log//${DUMP_INFO_FILENAME}

			echo "### ss -anpot ###" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
			ss -anpot >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
			echo -e "\n" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
			echo "### ss -anpou ###" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
			ss -anpou >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
			echo -e "\n" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
		fi

		echo "### ps -ef ###" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
		ps -ef >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
		echo -e "\n" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
	
		echo -e "\n" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
		echo -e "\n" >> ${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}
	
		if [ ${NUM} -ge ${COUNT} ] ;
		then
			break;
		fi
	
		sleep ${INTERVAL}
	
	done
	
	cp -p ${HINEMOS_HOME}/var/log/jvm_stdout.log ${HINEMOS_HOME}/var/log/${DUMP_FILENAME}
	
	echo -e "\n"
	echo "### Please get this file ###"
	echo "${HINEMOS_HOME}/var/log/${DUMP_FILENAME}"
	echo "${HINEMOS_HOME}/var/log/${DUMP_INFO_FILENAME}"
	echo -e "\n"

} # function threaddump


function log {

	DIR_NAME="hinemos_manager_log.${FILENAME_POSTFIX}"
	mkdir ${HINEMOS_HOME}/${DIR_NAME}
	
	echo "### /var/log/messages ###"
	cd /var
	ls log/ | grep messages*
	tar cpzf ${HINEMOS_HOME}/${DIR_NAME}/var_log_messages.tar.gz ./log/messages*
	echo -e "\n"
	
	echo "### ${HINEMOS_HOME}/var/log ###"
	cd ${HINEMOS_HOME}/var/
	ls log/
	tar cpzf ${HINEMOS_HOME}/${DIR_NAME}/hinemos_manager_log.tar.gz ./log/*
	echo -e "\n"
	
	echo "### create tarball ###"
	cd ${HINEMOS_HOME}
	tar cpzf ${HINEMOS_HOME}/${DIR_NAME}.tar.gz ./${DIR_NAME}/*
	
	rm -rf ${HINEMOS_HOME}/${DIR_NAME}
	
	echo ${HINEMOS_HOME}/${DIR_NAME}.tar.gz
	
	echo -e "\n"

} # funcion log

function diskspacecheck {

	echo "### file system disk space usage ###"
	df -h
	echo -e "\n"
	
	echo "### log directory disk space usage ###"
	USAGE=`du -hs /opt/hinemos/var/log/`
	echo ${USAGE}
	echo -e "\n"
	
	
	if expr "${USAGE}" : ".*G.*\/opt\/hinemos\/var\/log\/$" > /dev/null;  then
		echo "Disk space used by Hinemos Manager's Log(/opt/hinemos/var/log/) more than 1GByte."
		echo "Please delete log, before getting log about Hinemos Manager."
		exit 1
	fi
	
	
	echo "### Hinemos log archive output directory ###"
	echo ${HINEMOS_HOME}/
	echo -e "\n"
	
	while [ 1 ]
	do
		sleep 0.5 && read -p "Is it OK to collect Hinemos Manager's log ? (Y/N default:Y)" INPUT
		INPUT=${INPUT:-Y}
		case ${INPUT} in
			y|Y)
				break
				;;
			n|N)
				echo "Canceled."
				exit 1
				;;
			*)
				echo "Your input is invalid. Please input again."
				;;
		esac
	done
	
	echo -e "\n"

} # function diskspacecheck


########################################
# SHELL
########################################

# check parameters
if [ $# -eq 0 ]; then
	usage
	exit 1
fi
for OPT in $@; do
	case $OPT in
		--help|-h)
			usage
			exit 0
			;;
	esac
done

MODE=
# check options
while getopts ":w:u:p:" OPT; do
	case $OPT in
		w)
			export PGPASSWORD=${OPTARG}
			;;
		u)
			JMX_USER=${OPTARG}
			;;
		p)
			JMX_PASS=${OPTARG}
			;;
		*)
			MODE="${MODE}${OPTARG}"
	esac
done

shift $(( $OPTIND - 1 ))

echo ""
echo ""
echo "#################################################"
echo "### Hinemos stats collect script (20150501)   ###"
echo "###    Script start at `date '+%Y-%m-%d %T'`    ###"
echo "#################################################"
echo ""
case "-$MODE" in
	-v)
		hinemos
		;;
	-vv)
		hinemos
		os
		;;
	-vvv)
		hinemos
		os
		detail
		;;
	-o)
		diskspacecheck
		hinemos
		os
		detail
		log
		exit 0
		;;
	-t)
		threaddump
		exit 0
		;;
	-md5)
		md5sum
		exit 0
		;;
	*)
		ExitIllegalArgumentError
esac


echo "### Hinemos Manager Summary ###"
echo ${SUMMARY_FILE}

echo ""
echo "#################################################"
echo "###     Script end at `date '+%Y-%m-%d %T'`     ###"
echo "#################################################"

exit 0
